/*
 * Created on Aug 21, 2003
 */
package isac.bridge;

/**
 * @author rgradisc
 */

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.log4j.Logger;

/**
 * Clients2KernelServer: runs in its own thread and handles 
 * the communication from clients to the kernel 
 * @author rgradisc
 */
public class Clients2KernelServer implements Runnable {
  private ServerSocket server = null;
  private BridgeMain bridge;
  
  static Logger logger = Logger.getLogger(Clients2KernelServer.class.getName());

  /**
  * Constructor: create a new Clients2KernelServer
  * @param bridge
  */
  public Clients2KernelServer(BridgeMain bridge) {
  	logger.warn("Clients2KernelServer: bridge="+bridge);
    this.bridge = bridge;
  } 

  /**
  * Listen on socket: Wait for clients to connect to bridge,
  * and once the connect, start a new ClientOutputWorker that
  * sends the XML-output back to the client
  */
  public void listenSocket() {
    ClientOutputWorker w;
    Socket socket;
    try {
      server = new ServerSocket(bridge.getSocketPort());
    } catch (IOException e) {
      bridge.log(1, "Could not listen on port " + bridge.getSocketPort());
	  System.out.println("Could not listen on port " + bridge.getSocketPort());
      System.exit(-1);
    }
    while (true) {
      try {
        //TODO id should be fetched from the BridgeMain: 
        //smallestFreeBridgeRmiID
        //(so far: only one BridgeRMI)
        int id = 1;        
        socket = server.accept(); // wait for client connections
        bridge.log(1, "connection received");
        w = new ClientOutputWorker(id, socket, bridge);
        Thread t = new Thread(w, "java_bridge_clientworker" + id);
        t.setDaemon(true);
        t.start();
        bridge.log(1, "new Client accepted, id=" + id);
        //TODO if to much load, start new kernel: e.g. bridge.startNewKernel();        

      } catch (IOException e) {
        bridge.log(1, "Accept failed on port" + bridge.getSocketPort());
        System.out.println("Accept failed on port" + bridge.getSocketPort());
        //Quit program: could not connect to port
        System.exit(-1);
      }
    }
  }

  /**
   * This starts the Client2KernelServer thread
   */
  public void run() {
    listenSocket();
    try {
      bridge.log(1, "Clients2KernelServer: Cleaning up...");
      server.close();

    } catch (IOException e) {
      bridge.log(1, "Could not close socket");
      System.exit(-1);
    }
    bridge.log(1, "Done cleaning up");
  }
}
