/*
 * Created on Aug 21, 2003
 */
package isac.bridge;

/**
 * @author rgradisc
 *
 * Stores times of requests to sml-kernel.
 * This is used for informing Clients2KernelServer to 
 * kill sml-kernel if no response is coming back
 * for a certain amount of time, which means that
 * the bridge has probably crashed or is caught
 * in an endless loop, and neeeds to be restarted.
 */

import java.util.Vector;

import org.apache.log4j.Logger;

public class TimeCheckerThread extends Thread {

  //Time to wait for response from the kernel to a request  
  private long waitMillis;  
  //Stores the times when the waitTime for current request will run out
  private Vector timeOutTimes; 
  //Reference to the bridgeMain object
  private BridgeMain bridge;
  
  static Logger logger = Logger.getLogger(TimeCheckerThread.class.getName());
  /**
   * Constructor: instantiate a new TimeCheckerThread
   * @param bridge BridgeMain object that belongs to this TimeCheckerThread
   * @param waitMillis Time to wait for response from kernel before restarting
   */
  public TimeCheckerThread(BridgeMain bridge, long waitMillis) {
  	logger.warn("TimeCheckerThread: bridge="+bridge+", waitMillis"+waitMillis);
    this.bridge = bridge;
    this.timeOutTimes = bridge.getTimeOutTimes();  
    this.waitMillis = waitMillis;
  }

  /**
   * Start this Thread
   */
  public void run() {
    try {
      while (true) {
        if (!timeOutTimes.isEmpty()) {
          long first = ((Long) timeOutTimes.firstElement()).longValue();
          long timeOut = first + waitMillis;
          long now = System.currentTimeMillis();
          if (now > timeOut) {
            // timeOut: no response from sml-kernel within TIME_OUT: restart sml-kernel
            bridge.restartSML();
          } else {
            // sleep until first time in vector would run out (plus a little buffer)
            long sleepTime = timeOut - now + 20;
            bridge.log(
              1,
              "TimeCheckerThread: "
                + timeOutTimes.size()
                + " request(s) waiting for response, sleeping for "
                + sleepTime
                + " millis");
            sleep(sleepTime);
          }
        } else {
          // TimesVector is empty: No pending requests
          bridge.log(
            1,
            "TimeCheckerThread: no requests, sleeping for "
              + waitMillis
              + " millis");
          sleep(waitMillis);
        }
      }
    } catch (InterruptedException e) {
      bridge.log(1, "TimeCheckerThread interrupted");
    }
  }

}
