/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.calcheadviews;

import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadCompoundID;
import isac.util.formulae.CalcHeadItem;
import isac.util.formulae.CalcHeadItemList;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.Specification;

import java.awt.Color;
import java.util.*;

import javax.swing.*;
import javax.swing.text.*;

/**
 * Abstract class that specifies the commons of all different views of a 
 * <code>CalcHead</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 *
 */
public abstract class CalcHeadView extends JPanel implements CHeadView {
  protected JLabel labelGiven;
  protected JLabel labelWhere;
  protected JLabel labelFind;
  protected JLabel labelTheory;
  protected JLabel labelMethod;
  protected JLabel labelProblem;
  protected JLabel labelRelate;
  
  protected JTextPane txtGiven;
  protected JTextPane txtWhere;
  protected JTextPane txtFind;
  protected JTextPane txtTheory;
  protected JTextPane txtMethod;
  protected JTextPane txtProblem;
  protected JTextPane txtRelate;
  
  protected Locale locale;
  protected ResourceBundle resBundle;  
  
  public CalcHeadView(Locale locale) {
    this.locale = locale;
    resBundle = ResourceBundle.getBundle("CalcHeadPanel", 
                                                         locale);
  }
  
  /**
   * @param correct If true then colorize the label green otherwise red.
   * @param labelGiven Label that should be colored
   */
  protected void colorizeLabel(boolean correct, JLabel label) {
    if (correct)
      label.setForeground(new Color(40, 210, 40));
    else
      label.setForeground(Color.RED);    
  }

  /** @return False if a status was not CORRECT 
   * 
   * WN040908 part of undesired behaviour: 
   * the status (below 'attributes') of items should be displayed
   * _per item_ and not for some collection (eg. for 'given')
   * */
  protected boolean checkAndFormatCalcHeadItems(Iterator it, JTextPane txt) {
    boolean correct = true;
    StyledDocument doc = txt.getStyledDocument();
    while (it.hasNext()) {
      Object o = it.next();
      try {
        if (o instanceof CalcHeadItem) {
          CalcHeadItem chi = (CalcHeadItem)o;

          SimpleAttributeSet attributes = new SimpleAttributeSet();
          if (chi.getStatus() != CalcHead.CALCHEAD_CORRECT) {
            StyleConstants.setForeground(attributes, Color.red);
            correct = false;
          }
          doc.insertString(doc.getLength(), chi.toSMLString(), attributes);
        } else if (o instanceof String) {
          String s = (String)o;
          doc.insertString(doc.getLength(), s, null);
        }
        if (it.hasNext())
          doc.insertString(doc.getLength(), ",", null);
      } catch (BadLocationException e) {
        e.printStackTrace();
      }
    }
    return correct;
  }
  
  /** Clear all strings in the text fields  */
  protected void clearTextFields() {
    if (txtGiven != null)
      txtGiven.setText("");    
    if (txtWhere != null)
      txtWhere.setText("");
    if (txtFind != null)
      txtFind.setText("");
    if (txtRelate != null)
        txtRelate.setText("");        
    if (txtTheory != null)
      txtTheory.setText("");
    if (txtProblem != null)
      txtProblem.setText("");
    if (txtMethod != null)
      txtMethod.setText("");
    //WN040908 missing: belongs-to 
  }
  

  /**
   * parse the Text Field txtGiven which can have multiple mathTexts seperated
   * by a seperator char(a comma at the moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 10:49:30
   */
  protected String[] parseTxtGivenToStringArray(char seperator) {
    String given = txtGiven.getText();
    String seperatorString = String.valueOf(seperator);
    return given.split(seperatorString);  
  }
  
  /**
   * this method inserts the mathTexts from the TextField txtGiven in the
   * calcHead
   * @param calcHead is modified in the method!
   * @author Alois Kirchsteiger 10.09.2004 ~12
   */
  protected void insertTxtGivenInCalcHead(CalcHead calcHead) {
    String[] given = parseTxtGivenToStringArray(',');
    CalcHeadItemList givenList = new CalcHeadItemList();
    for (int i=0; i<given.length; i++) {
      CalcHeadItem givenItem = new CalcHeadItem();
      givenItem.setText(given[i]);
      givenList.addItem(givenItem);
    }
    calcHead.setGiven(givenList);
  }
  
  /**
   * parse the Text Field txtWhere which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtWhereToStringArray(char seperator) {
    String where = txtWhere.getText();
    String seperatorString = String.valueOf(seperator);
    return where.split(seperatorString);  
  }
  
  /**
   * this method inserts the mathTexts from the TextField txtWhere in the
   * calcHead
   * @param calcHead is modified in the method!
   * @author Alois Kirchsteiger 10.09.2004 ~12
   */
  protected void insertTxtWhereInCalcHead(CalcHead calcHead) {
    String[] where = parseTxtWhereToStringArray(',');
    CalcHeadItemList whereList = new CalcHeadItemList();
    for (int i=0; i<where.length; i++) {
      CalcHeadItem whereItem = new CalcHeadItem();
      whereItem.setText(where[i]);
      whereList.addItem(whereItem);
    }
    calcHead.setGiven(whereList);
  }
  
  
  /**
   * parse the Text Field txtFind which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtFindToStringArray(char seperator) {
    String find = txtFind.getText();
    String seperatorString = String.valueOf(seperator);
    return find.split(seperatorString);  
  }
  
  /**
   * this method inserts the mathTexts from the TextField txtFind in the
   * calcHead
   * @param calcHead is modified in the method!
   * @author Alois Kirchsteiger 10.09.2004 ~12
   */
  protected void insertTxtFindInCalcHead(CalcHead calcHead) {
    String[] find = parseTxtFindToStringArray(',');
    CalcHeadItemList findList = new CalcHeadItemList();
    for (int i=0; i<find.length; i++) {
      CalcHeadItem findItem = new CalcHeadItem();
      findItem.setText(find[i]);
      findList.addItem(findItem);
    }
    calcHead.setGiven(findList);
  }
  
  /**
   * parse the Text Field txtRelate which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtRelateToStringArray(char seperator) {
    String relate = txtRelate.getText();
    String seperatorString = String.valueOf(seperator);
    return relate.split(seperatorString);  
  }
  
  /**
   * this method inserts the mathTexts from the TextField txtRelate in the
   * calcHead
   * @param calcHead is modified in the method!
   * @author Alois Kirchsteiger 10.09.2004 ~12
   */
  protected void insertTxtRelateInCalcHead(CalcHead calcHead) {
    String[] relate = parseTxtRelateToStringArray(',');
    CalcHeadItemList relateList = new CalcHeadItemList();
    for (int i=0; i<relate.length; i++) {
      CalcHeadItem relateItem = new CalcHeadItem();
      relateItem.setText(relate[i]);
      relateList.addItem(relateItem);
    }
    calcHead.setGiven(relateList);
  }
  
  
  /**
   * parse the Text Field txtProblem which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtProblemToStringArray(char seperator) {
    String problem = txtProblem.getText();
    String seperatorString = String.valueOf(seperator);
    return problem.split(seperatorString);  
  }
  
  /**
   * insert the mathTexts from the TextField txtProblem in the CalcHead
   * @param calcHead
   */
  protected void insertTxtProblemInCalcHeadSpecification(CalcHead calcHead) {
    Specification specification = calcHead.getSpecification();
    String[] problem = parseTxtProblemToStringArray(',');
    CalcHeadCompoundID problemCalcCompoundID = new CalcHeadCompoundID();
    for (int i=0; i<problem.length; i++) {
      problemCalcCompoundID.addString(problem[i]);
    }
    specification.setProblem(problemCalcCompoundID);
    calcHead.setSpecification(specification);
  }
  
  /**
   * parse the Text Field txtMethod which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtMethodToStringArray(char seperator) {
    String method = txtMethod.getText();
    String seperatorString = String.valueOf(seperator);
    return method.split(seperatorString);  
  }
  
  /**
   * insert the mathTexts from the TextField txtMethod in the CalcHead
   * @param calcHead
   */
  protected void insertTxtMethodInCalcHeadSpecification(CalcHead calcHead) {
    Specification specification = calcHead.getSpecification();
    String[] method = parseTxtMethodToStringArray(',');
    CalcHeadCompoundID methodCalcCompoundID = new CalcHeadCompoundID();
    for (int i=0; i<method.length; i++) {
      methodCalcCompoundID.addString(method[i]);
    }
    specification.setProblem(methodCalcCompoundID);
    calcHead.setSpecification(specification);
  }
  
  /**
   * parse the Text Field txtTheory which can have multiple mathTexts seperated
   * by a seperator char(a comma at te moment) so that as a result those multiple mathTexts are in a
   * String[] where every element is one mathText 
   * 
   * @param seperator: the char that seperates multiple mathTexts
   * @return String[] which contains all mathTexts of the text Field
   * @author Alois Kirchsteiger 10.09.2004 ~11
   */
  protected String[] parseTxtTheoryToStringArray(char seperator) {
    String theory = txtTheory.getText();
    String seperatorString = String.valueOf(seperator);
    return theory.split(seperatorString);  
  }
  
  /**
   * insert the mathTexts from the TextField txtTheory in the CalcHead
   * @param calcHead
   */
  protected void insertTxtTheoryInCalcHeadSpecification(CalcHead calcHead) {
    Specification specification = calcHead.getSpecification();
    String theory = txtTheory.getText();
    CalcHeadSimpleID theoryCalcHeadSimpleID = new CalcHeadSimpleID();
    theoryCalcHeadSimpleID.setID(theory);
    specification.setTheory(theoryCalcHeadSimpleID);
    calcHead.setSpecification(specification);
  }
  
  /**
   * inserts the whle specification (theory, method, problem) in the calcHead
   * @param calcHead
   * @author Alois Kirchsteiger 10.09.2004 13:56:25
   */
  
  protected void insertSpecificationFromTextFieldsInCalcHead(CalcHead calcHead) {
    insertTxtTheoryInCalcHeadSpecification(calcHead);
    insertTxtMethodInCalcHeadSpecification(calcHead);
    insertTxtProblemInCalcHeadSpecification(calcHead);
  }
}
