/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the example hierarchy and converts
 * it to a object representation.
 * Each NODE Tag is converted to a <code>ExampleNode</code> object. The
 * example nodes are managed by the <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 */
public class XMLExampleParser extends XMLHierarchyParser {
  
  protected static final String TYPE_EXAMPLE = "EXAMPLE";

  public XMLExampleParser(String xml) {
    super(xml);
    System.out.println("wn##### XMLExampleParser, XML="+xml);
  }
//  public NodeList getChilds(Node node) {
//    return node.getChildNodes();
//  }
//  
  private void createExampleNodesHierarchy(Node node, ExampleNode root, 
                                           String txtType) {
    ExampleNode exNode = null;
    String elemName = null;
    // is there anything to do?
    if (node == null) {
      return;
    }

    int type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE : {
          Document document = (Document)node;
          createExampleNodesHierarchy(document.getDocumentElement());
          break;
      }

      case Node.ELEMENT_NODE :  {
          String nodeName = node.getNodeName().trim(); 
          if (nodeName.equals(TAG_NODE)) {
            exNode = new ExampleNode();
            hiearchy.addNode(root, exNode);
            // System.out.println(node.getNodeName().trim()); 
          } else if (nodeName.equals(TYPE_ID)){
            elemName = TYPE_ID;
          } else if (nodeName.equals(TYPE_CONTENTREF)) {
            elemName = TYPE_CONTENTREF;
          } else if (nodeName.equals(TYPE_EXAMPLE)) {
            elemName = TYPE_EXAMPLE;
          }
          
          // drop through to entity reference
      }

      case Node.ENTITY_REFERENCE_NODE :  {
          Node child = node.getFirstChild();
          while (child != null) {
            // elemName is set -> next node is of type text
            if (elemName != null)
              createExampleNodesHierarchy(child, root, elemName);
            createExampleNodesHierarchy(child, exNode, txtType);
            child = child.getNextSibling();
          }
          break;
          
      }

      case Node.CDATA_SECTION_NODE : {
        break;
      }

      case Node.TEXT_NODE :  {
          // unknown elemType or whitespace is ignored
          if (txtType == null)
            return;
             
          Text text = (Text)node;
          String txt = text.getNodeValue().trim();
          
          if (txtType.equals(TYPE_ID)) {
            root.setId(txt);
            // System.out.println("ID: " + txt);
          } else if (txtType.equals(TYPE_CONTENTREF)) {
            root.setContentref(txt);
            // System.out.println("CONTENTREF: " +txt); 
          } else if (txtType.equals(TYPE_EXAMPLE)) {
            root.setExample(txt);
            // System.out.println("EXAMPLE: " + txt);
          }
          break;
      }
    }
  }
  
  public Hierarchy createExampleNodesHierarchy(Node node) {
    this.createExampleNodesHierarchy(node, null, null);
    return hiearchy;
  }
}
