/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the method hierarchy and converts
 * it to a object representation.
 * Each NODE Tag is converted to a <code>MethodNode</code> object. The
 * method nodes are managed by the <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 *
 */
public class XMLMethodParser extends XMLHierarchyParser {
  
  protected static final String TYPE_NO = "NO";
  
  public XMLMethodParser(String xml) {
    super(xml);
    System.out.println("WN##### XMLMethodParser, xml="+xml);
  }
  
  private void createMethodNodesHierarchy(Node node, MethodNode root, 
                                           String txtType) {
    MethodNode methodNode = null;
    String elemName = null;
    // is there anything to do?
    if (node == null) {
      return;
    }

    int type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE : {
          Document document = (Document)node;
          createMethodNodesHierarchy(document.getDocumentElement());
          break;
      }

      case Node.ELEMENT_NODE :  {
          String nodeName = node.getNodeName().trim(); 
          if (nodeName.equals(TAG_NODE)) {
            methodNode = new MethodNode();
            hiearchy.addNode(root, methodNode);
            // System.out.println(node.getNodeName().trim()); 
          } else if (nodeName.equals(TYPE_ID)){
            elemName = TYPE_ID;
          } else if (nodeName.equals(TYPE_CONTENTREF)) {
            elemName = TYPE_CONTENTREF;
          } else if (nodeName.equals(TYPE_NO)) {
            elemName = TYPE_NO;
          }          
          // drop through to entity reference
      }

      case Node.ENTITY_REFERENCE_NODE :  {
          Node child = node.getFirstChild();
          while (child != null) {
            // elemName is set -> next node is of type text
            if (elemName != null)
              createMethodNodesHierarchy(child, root, elemName);
            createMethodNodesHierarchy(child, methodNode, txtType);
            child = child.getNextSibling();
          }
          break;
          
      }

      case Node.CDATA_SECTION_NODE : {
        break;
      }

      case Node.TEXT_NODE :  {
          // unknown elemType or whitespace is ignored
          if (txtType == null)
            return;
             
          Text text = (Text)node;
          String txt = text.getNodeValue().trim();
          
          if (txtType.equals(TYPE_ID)) {
            root.setId(txt);
            // System.out.println("ID: " + txt);
          } else if (txtType.equals(TYPE_CONTENTREF)) {
            root.setContentref(txt);
            // System.out.println("CONTENTREF: " +txt); 
          } else if (txtType.endsWith(TYPE_NO)) {
            root.setNo(txt);
           // System.out.println("NO: " + txt);
          }
          break;
      }
    }
  }
  
  public Hierarchy createMethodNodesHierarchy(Node node) {
    this.createMethodNodesHierarchy(node, null, null);
    return hiearchy;
  }  
}
