/*
 * Created on Oct 6, 2003
 */
package isac.session;


import isac.browserdialog.BDialog;
import isac.wsdialog.DGuide;

import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

/**
 * @author gismo
 */
public class ObjectManager extends UnicastRemoteObject implements OManager {
	SessionDialog sdialog_=SessionDialog.getInstance();
	/**
	 * instantiates the Objectmanager which is used to simulate a function of dinopolis
	 * The ObjectManager provides functions to fetch objects which are used by other modules
	 * of the system using RMI
	 * @throws RemoteException
	 */
	protected ObjectManager() throws RemoteException {
		super();
		rmiBind();
	}

/**
 *  fetch the BrowserDialog identified by <code>bdialog_id</code>
 */
	public BDialog getBrowserDialog(String bdialog_id) throws RemoteException {
		return sdialog_.getBDialog(bdialog_id);
	}

/**
 * fetch the WorksheetDialog identified by <code>wdialog_id</code>
 */ 
	public DGuide getWorksheetDialog (String wdialog_id) throws RemoteException{
		return (DGuide) sdialog_.getDGuide(wdialog_id);
	}

/**
 * fetch the SessionDialog of the system - multiple calls to this method result in one and 
 * the same SessionDialog.
 */
	public SDialog getSessionDialog() throws RemoteException{
			if(sdialog_==null)
				sdialog_=SessionDialog.getInstance();
			return sdialog_;
	}

/*
 * instantiates the ObjectManager and registers it on the RMIRegistry 
 * on the local machine as //localhost/isac-ObjectManager . 
 * If no RMIRegistry is running, a new one is started.
 */
	public static void main(String[] args) {
		try {
			new ObjectManager();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	private void rmiBind(){
		if(System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		String name = "//localhost/isac-ObjectManager";
		try {
			System.out.println("try to bind as "+name);
			Naming.rebind(name, this);
			System.out.println("Object Manager bound to "+name);
		} catch (java.rmi.ConnectException e) {	    
		System.err.println("failed to contact as "+name+
			" (creating RMI-Server on localhost:1099)");
		try{
		LocateRegistry.createRegistry(1099);
		rmiBind();
		}catch(java.rmi.RemoteException exc2){
		  System.err.println("can not create registry:"+exc2.getMessage());
		}
		} catch (Exception exc){ 
		System.err.println("failed to connect to rmiregistry");
		System.err.println(exc);
	}
	}
	
}
