/*
 * Created on Feb 22, 2005
 *
* @author Alan Krempler
 */
 
 
package isac.util.usersettings;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;


/**
 * Class for storing/retrieving user settings, i.e. settings which can be set manually by the user
 * as opposed to the {@link isac.util.usersettings.UserModel#}, which is an abstraction of the system's expeience with a user
 * and canot be altered manually. 
 * As there is no fixed set of user settings, you can regard this class as a simple storage backend for key/value pairs.
 * @author Alan Krempler
 *
 */
public class UserSettings implements Serializable{
	
	private HashMap storage_;
	
	UserSettings(){
		storage_=new HashMap();
	}
	/**
	 * @param key	A string key
	 * @return	The string value associated with the key parameter; null, if nothing is stored under this key
	 */
	public String getValue(String key) {
		return (String)storage_.get(key);
	}
	
	/**
	 * @param key	The key under which "value" is to be stored. Existing data unde the same key will be replaced.
	 * @param value	The value to be stored under "key"
	 * @return	true in case of success, false in case of error, for example if trying to replace stored values without
	 * proper permissions
	 */
	public boolean setValue(String key, String value) {
		storage_.put(key, value);
		return true;
	}

	
	

}
