/*
 * Created on Dec 12, 2003
 */
package isac.wsdialog;

/**
 * @author Alan Krempler
 *
 * Constants describing interactions or request from the user
 */
public interface IUIElement {
	
	/******************* NOTIFICATIONS: SPECIFYING PHASE ***********************/
	
	public static final int UI_DUMMY_FIRST_SPECIFY = 1;
	/**
	 * During the specifying phase,
	 * try to match a problem type to the data entered so far
	 * WN                     ////    ?items of the model entered so far
	 */
	public static final int UI_SPECIFY_TRY_MATCH = 1;

	/**
	 * During the specifying phase,
	 * try to refine the problem type.
	 * WN                        //// 
	 */
	public static final int UI_SPECIFY_TRY_REFINE = 2;
	
	/**
	 * During the specifying phase,
	 * change the view on the CalcHead.  
	 */
	public static final int UI_SPECIFY_CHANGE_VIEW = 3;

	/**
	 * During the specifying phase,
	 * have the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_CALCHEAD = 4;

	/**
	 * During the specifying phase,
	 * have the Method field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_METHOD = 5;

	/**
	 * During the specifying phase,
	 * have the Theory field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_THEORY = 6;

	/**
	 * During the specifying phase,
	 * have the Problem field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_PROBLEM = 7;

	/**
	 * During the specifying phase,
	 * have the Given field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_GIVEN = 8;

	/**
	 * During the specifying phase,
	 * have the Find field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_FIND = 9;
	
	/**
	 * During the specifying phase,
	 * have the Relate field of the CalcHead completed by the math engine.   
	 */
	public static final int UI_SPECIFY_COMPLETE_RELATE = 10;

	/**
	 * During the specifying phase,
	 * have the CalcHead checked for completeness and consistency.   
	 */
	public static final int UI_SPECIFY_CHECK_CALCHEAD = 11;
	
	/**
	 * Leave the specifying phase, enter the solving phase
	 * and calculate the first step of the mathod.   
	 */
	public static final int UI_SPECIFY_CALCULATE_1 = 12;
	
	/**
	 * Leave the specifying phase, enter the solving phase
	 * and calculate until a final result is reached.   
	 */
	public static final int UI_SPECIFY_CALCULATE_ALL = 13;

	public static final int UI_DUMMY_LAST_SPECIFY = 13;
	
	
	/******************* NOTIFICATIONS: SOLVING PHASE ***********************/

	public static final int UI_DUMMY_FIRST_SOLVE = 21;

	/**
	 * During the solving phase,
	 * calculate one more step.   
	 */
	public static final int UI_SOLVE_CALCULATE_1 = 21;
	
	/**
	 * During the solving phase,
	 * calculate until a final result is reached.   
	 */
	public static final int UI_SOLVE_CALCULATE_ALL = 22;

	/**
	 * During the solving phase,
	 * calculate until the current subproblem is solved.   
	 */
	public static final int UI_SOLVE_CALCULATE_SUBPROBLEM = 23;

	/**
	 * During the solving phase,
	 * request editing the currently active formula.
	 * 
	 * WN0412 If request is conceded, do @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE   
	 */
	public static final int UI_SOLVE_EDIT_ACTIVE_FORMULA = 24;

	/**
	 * During the solving phase,
	 * notify that editing the currently active formula is finished.
	 * This implies a request for updating the CalcTree. 
	 */	
	public static final int UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE = 25;
	
	/**
	 * During the solving phase,
	 * request inserting a new formula _after_ the currently active
	 * formula, making the new formula the currently active formula.
	 * At present, this does not imply editing the formula and must
	 * be requested in a separate step.  
	 * WN0412: ...??? is this right interpreted by ...
	 * WN0412 If request is conceded, do @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE   
	 */	
	public static final int UI_SOLVE_APPEND_USER_FORMULA = 26;
	
	/**
	 * During the solving phase,
	 * request making the referened formula the currently active
	 * formula.
	 */		
	public static final int UI_SOLVE_MOVE_ACTIVE_FORMULA = 27;

	/**
	 * During the solving phase,
	 * ask for the tactic the math engine would apply to the current formula.
	 */	
	public static final int UI_SOLVE_GET_PROPOSED_TACTIC = 30;

	/**
	 * During the solving phase,
	 * ask for a list of tactics to the current formula.
	 */	
	public static final int UI_SOLVE_GET_APPLICABLE_TACTICS = 31;
	
	/**
	 * During the solving phase,
	 * set the tactic to be applied to the current formula.
	 */	
	public static final int UI_SOLVE_SET_NEXT_TACTIC = 32;
	
	/**
	 * When entering a formula during the solving phase,
	 * request help on entering the formula.
	 */		
	public static final int UI_SOLVE_HELP_ENTERING_FORMULA = 33;

	/**
	 * During the solving phase,
	 * show which assumptions hold at the current point of calculation.
	 */	
	public static final int UI_SOLVE_SHOW_ASSUMPTIONS = 40;
	
	/**
	 * During the solving phase,
	 * request more detail about the currently referenced element.
	 */		
	public static final int UI_SOLVE_SHOW_DETAILS = 41;
	
	public static final int UI_DUMMY_LAST_SOLVE = 41;
	
	/******************* NOTIFICATIONS: unaware of PHASE *****************/

	public static final int UI_CALCULATE_1 = 101;

	public static final int UI_CALCULATE_ALL = 102;

	
	/******************* ACTIVE REQUESTS TO THE UI ***********************/

	/**
	 * Request editing the currently active formula.
	 * DG distinguishes between SPECIFY and SOLVE;
	 * thus could drop UI_SOLVE_EDIT_ACTIVE_FORMULA : int
	 */
	public static final int UI_DO_EDIT_FORMULA = 129;

	/**
	 * Request appending a new, empty formula.
	 */	
	public static final int UI_DO_APPEND_FORMULA = 130;
	
	/**
	 * Activate an UI element.
	 */
	public static final int UI_DO_ACTIVATE = 131;
	
	/**
	 * Deactivate an UI element.
	 */
	public static final int UI_DO_DEACTIVATE = 132;
	
	/**
	 * The Dialog Guide has detached the listener and will
	 * not send any more control requests.
	 * If registered as a Data Change Listener, update notifications
	 * will still arrive!
	 */
	public static final int UI_DO_DETACH = 133; //WN ??? Design ???
}
